@@ -2,7 +2,10 @@
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 # Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
+# type: ignore # Deprecated module.
+
 import optparse  # pylint: disable=deprecated-module
+import warnings
 
 from pylint.config.option import Option
 
@@ -18,7 +21,13 @@ def _level_options(group, outputlevel):
 
 class OptionParser(optparse.OptionParser):
     def __init__(self, option_class, *args, **kwargs):
-        super().__init__(option_class=Option, *args, **kwargs)
+        # TODO: 3.0: Remove deprecated class
+        warnings.warn(
+            "OptionParser has been deprecated and will be removed in pylint 3.0",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        super().__init__(option_class=Option, *args, **kwargs)  # noqa: B026
 
     def format_option_help(self, formatter=None):
         if formatter is None:
@@ -40,7 +49,7 @@ class OptionParser(optparse.OptionParser):
         # Drop the last "\n", or the header if no options or option groups:
         return "".join(result[:-1])
 
-    def _match_long_opt(self, opt):
+    def _match_long_opt(self, opt):  # pragma: no cover # Unused
         """Disable abbreviations."""
         if opt not in self._long_opt:
             raise optparse.BadOptionError(opt)
