@@ -2,8 +2,11 @@
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 # Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
+from __future__ import annotations
+
 import warnings
-from typing import Any, NamedTuple, Optional, Sequence, Tuple, TypeVar, Union
+from collections.abc import Sequence
+from typing import Any, NamedTuple, TypeVar
 
 from astroid import nodes
 
@@ -12,74 +15,36 @@ from pylint.interfaces import UNDEFINED, Confidence
 from pylint.message.message import Message
 from pylint.testutils.constants import UPDATE_OPTION
 
-T = TypeVar("T")
+_T = TypeVar("_T")
 
 
 class MessageTest(NamedTuple):
     msg_id: str
-    line: Optional[int] = None
-    node: Optional[nodes.NodeNG] = None
-    args: Optional[Any] = None
-    confidence: Optional[Confidence] = UNDEFINED
-    col_offset: Optional[int] = None
-    end_line: Optional[int] = None
-    end_col_offset: Optional[int] = None
+    line: int | None = None
+    node: nodes.NodeNG | None = None
+    args: Any | None = None
+    confidence: Confidence | None = UNDEFINED
+    col_offset: int | None = None
+    end_line: int | None = None
+    end_col_offset: int | None = None
     """Used to test messages produced by pylint.
 
     Class name cannot start with Test as pytest doesn't allow constructors in test classes.
     """
 
 
-class MalformedOutputLineException(Exception):
-    def __init__(
-        self,
-        row: Union[Sequence[str], str],
-        exception: Exception,
-    ) -> None:
-        example = "msg-symbolic-name:42:27:MyClass.my_function:The message"
-        other_example = "msg-symbolic-name:7:42::The message"
-        expected = [
-            "symbol",
-            "line",
-            "column",
-            "end_line",
-            "end_column",
-            "MyClass.myFunction, (or '')",
-            "Message",
-            "confidence",
-        ]
-        reconstructed_row = ""
-        i = 0
-        try:
-            for i, column in enumerate(row):
-                reconstructed_row += f"\t{expected[i]}='{column}' ?\n"
-            for missing in expected[i + 1 :]:
-                reconstructed_row += f"\t{missing}= Nothing provided !\n"
-        except IndexError:
-            pass
-        raw = ":".join(row)
-        msg = f"""\
-{exception}
-
-Expected '{example}' or '{other_example}' but we got '{raw}':
-{reconstructed_row}
-
-Try updating it with: 'python tests/test_functional.py {UPDATE_OPTION}'"""
-        super().__init__(msg)
-
-
 class OutputLine(NamedTuple):
     symbol: str
     lineno: int
     column: int
-    end_lineno: Optional[int]
-    end_column: Optional[int]
+    end_lineno: int | None
+    end_column: int | None
     object: str
     msg: str
     confidence: str
 
     @classmethod
-    def from_msg(cls, msg: Message, check_endline: bool = True) -> "OutputLine":
+    def from_msg(cls, msg: Message, check_endline: bool = True) -> OutputLine:
         """Create an OutputLine from a Pylint Message."""
         column = cls._get_column(msg.column)
         end_line = cls._get_py38_none_value(msg.end_line, check_endline)
@@ -96,7 +61,7 @@ class OutputLine(NamedTuple):
         )
 
     @staticmethod
-    def _get_column(column: str) -> int:
+    def _get_column(column: str | int) -> int:
         """Handle column numbers except for python < 3.8.
 
         The ast parser in those versions doesn't return them.
@@ -107,9 +72,9 @@ class OutputLine(NamedTuple):
         return int(column)
 
     @staticmethod
-    def _get_py38_none_value(value: T, check_endline: bool) -> Optional[T]:
-        """Used to make end_line and end_column None as indicated by our version compared to
-        `min_pyver_end_position`.
+    def _get_py38_none_value(value: _T, check_endline: bool) -> _T | None:
+        """Used to make end_line and end_column None as indicated by our version
+        compared to `min_pyver_end_position`.
         """
         if not check_endline:
             return None  # pragma: no cover
@@ -117,13 +82,15 @@ class OutputLine(NamedTuple):
 
     @classmethod
     def from_csv(
-        cls, row: Union[Sequence[str], str], check_endline: bool = True
-    ) -> "OutputLine":
-        """Create an OutputLine from a comma separated list (the functional tests expected
-        output .txt files).
+        cls, row: Sequence[str] | str, check_endline: bool = True
+    ) -> OutputLine:
+        """Create an OutputLine from a comma separated list (the functional tests
+        expected output .txt files).
         """
         if isinstance(row, str):
             row = row.split(",")
+        # noinspection PyBroadException
+        # pylint: disable = too-many-try-statements
         try:
             column = cls._get_column(row[2])
             if len(row) == 5:
@@ -132,6 +99,7 @@ class OutputLine(NamedTuple):
                     "expected confidence level, expected end_line and expected end_column. "
                     "An OutputLine should thus have a length of 8.",
                     DeprecationWarning,
+                    stacklevel=2,
                 )
                 return cls(
                     row[0],
@@ -149,6 +117,7 @@ class OutputLine(NamedTuple):
                     "expected end_line and expected end_column. An OutputLine should thus have "
                     "a length of 8.",
                     DeprecationWarning,
+                    stacklevel=2,
                 )
                 return cls(
                     row[0], int(row[1]), column, None, None, row[3], row[4], row[5]
@@ -167,10 +136,16 @@ class OutputLine(NamedTuple):
                     row[7],
                 )
             raise IndexError
-        except Exception as e:
-            raise MalformedOutputLineException(row, e) from e
+        except Exception:  # pylint: disable=broad-except
+            warnings.warn(
+                "Expected 'msg-symbolic-name:42:27:MyClass.my_function:The message:"
+                f"CONFIDENCE' but we got '{':'.join(row)}'. Try updating the expected"
+                f" output with:\npython tests/test_functional.py {UPDATE_OPTION}",
+                UserWarning,
+            )
+            return cls("", 0, 0, None, None, "", "", "")
 
-    def to_csv(self) -> Tuple[str, str, str, str, str, str, str, str]:
+    def to_csv(self) -> tuple[str, str, str, str, str, str, str, str]:
         """Convert an OutputLine to a tuple of string to be written by a
         csv-writer.
         """
@@ -186,7 +161,7 @@ class OutputLine(NamedTuple):
         )
 
     @staticmethod
-    def _value_to_optional_int(value: Optional[str]) -> Optional[int]:
+    def _value_to_optional_int(value: str | None) -> int | None:
         """Checks if a (stringified) value should be None or a Python integer."""
         if value == "None" or not value:
             return None
